@@ -172,8 +172,8 @@ class BlockchainProtocol(LineReceiver):
                         self.factory.peer.blockchain.get_block_locator()])
                         
                     # Send all the pending transactions
-                    for transaction_hash in \
-                        self.factory.peer.blockchain.transactions.iterkeys():
+                    for transaction_hash, _ in \
+                        self.factory.peer.blockchain.get_transactions():
                         
                         self.send_message(["TXINV", bytes2string(
                             transaction_hash)])
@@ -206,8 +206,8 @@ class BlockchainProtocol(LineReceiver):
                         self.factory.peer.blockchain.get_block_locator()])
                         
                     # Send all the pending transactions
-                    for transaction_hash in \
-                        self.factory.peer.blockchain.transactions.iterkeys():
+                    for transaction_hash, _ in \
+                        self.factory.peer.blockchain.get_transactions():
                         
                         self.send_message(["TXINV", bytes2string(
                             transaction_hash)])
@@ -291,9 +291,11 @@ class BlockchainProtocol(LineReceiver):
                 # Make a Block object
                 block = Block.from_bytes(block_bytes)
                 
-                # Queue it
-                self.block_queue.append(block)
-                self.block_queued.add(block.block_hash())
+                if block.block_hash() not in self.block_queued:
+                    # Queue it if it's not already queued. Because of the set it
+                    # can only be queued once.
+                    self.block_queue.append(block)
+                    self.block_queued.add(block.block_hash())
                                 
             elif parts[0] == "TXINV":
                 # They have sent us a hash of a transaction that they have. If
@@ -340,7 +342,7 @@ class BlockchainProtocol(LineReceiver):
                     
                     # This is a legal transaction to accept from a peer (not
                     # something like a block reward).
-                    logging.info("Transaction acceptable from peer.")
+                    logging.debug("Transaction acceptable from peer.")
                 
                     # Give it to the blockchain as bytes. The blockchain can
                     # determine whether to forward it on or not and call the
