@@ -3,7 +3,7 @@ Peer.py: contains the Peer class.
 
 """
 
-import random, time, threading, logging
+import random, time, threading, logging, socket
 
 from twisted.internet import reactor, endpoints
 
@@ -67,8 +67,13 @@ class Peer(object):
         # Save the blockchain
         self.blockchain = blockchain
         
-        # Remember our external address (which may be None)
-        self.external_address = external_address
+        if external_address is not None:
+            # We have an external address. Make sure it's an address and
+            # remember it.
+            self.external_address = socket.gethostbyname(external_address)
+        else:
+            # No external address known.
+            self.external_address = None
         
         # Remember our port. We may need to tell people about it if we connect
         # to them.
@@ -97,9 +102,12 @@ class Peer(object):
         self.known_peers = sqliteshelf.SQLiteShelf(peer_file, table="peers", 
             lazy=True)
         
-        # Keep a set of hostnames we have open, outgoing connections to
+        # Keep a set of IPs we have open, outgoing connections to
         self.outgoing_hosts = set()
         
+        # And one of IPs we have active, incomming connections from
+        self.incoming_hosts = set()
+        
         # Keep a set of open connections (Protocol objects)
         self.connections = set()
         
@@ -167,6 +175,11 @@ class Peer(object):
             else:
                 # Add the connection to our list of connections
                 self.connections.add(connection)
+                
+                if connection.incoming:
+                    # Remember the host connecting in to us
+                    self.incoming_hosts.add(connection.transport.getPeer()[0])
+                
                 return True
         
     def lost_connection(self, connection):
@@ -179,6 +192,10 @@ class Peer(object):
         with self.lock:
             # Drop the connection from our list of connections
             self.connections.discard(connection)
+            
+            if connection.incoming:
+                # This was a connection we got, so throw it out of that set.
+                self.incoming_hosts.discard(connection.transport.getPeer()[0])
         
     
     def lost_outgoing_connection(self, host):
@@ -215,6 +232,10 @@ class Peer(object):
             if time_seen > int(time.time()) + 10 * 60:
                 # Don't accept peers from more than 10 minutes into the future.
                 return
+                
+            # Normalize host to an IPv4 address. TODO: do something to recognize
+            # IPv6 addresses and pass them unchanged.
+            host = socket.gethostbyname(host)
         
             if self.known_peers.has_key(host):
                 # We know of this peer, but we perhaps ought to update the last
@@ -331,9 +352,11 @@ class Peer(object):
                         host = random.sample(self.known_peers, 1)[0]
                         
                         # Try at most 100 times to find a host we aren't
-                        # connected to
+                        # connected to, and which isn't trivially obviously us.
                         attempt = 1
-                        while host in self.outgoing_hosts and attempt < 100:
+                        while (host in self.outgoing_hosts or 
+                            host in self.incoming_hosts or 
+                            host == self.external_address) and attempt < 100:
                             # Try a new host
                             host = random.sample(self.known_peers, 1)[0]
                             
@@ -408,7 +431,8 @@ class Peer(object):
             # Keep track of how long this takes.
             save_start = time.clock()
             
-            pybc.science.start_timer("sync")
+            if self.tick_count % 60 == 0:
+                pybc.science.start_timer("sync")
             
             # Sync the blockchain to disk. This needs to lock the blockchain, so
             # the blockchain must never wait on the peer while holding its lock.
@@ -419,7 +443,8 @@ class Peer(object):
             # Sync the known peers to disk
             self.known_peers.sync()
             
-            pybc.science.stop_timer("sync")
+            if self.tick_count % 60 == 0:
+                pybc.science.stop_timer("sync")
             
             # How long did it take?
             save_end = time.clock()
