@@ -564,10 +564,6 @@ class CoinState(State):
             # Get the hash of the transaction
             transaction_hash = transaction.transaction_hash()
             
-            logging.debug("Applying transaction with {} inputs and {} "
-                "outputs".format(len(transaction.inputs),
-                len(transaction.outputs)))
-            
             self.apply_transaction(transaction)
             
         if self.get_hash() != block.state_hash:
@@ -599,10 +595,6 @@ class CoinState(State):
             # Get the hash of the transaction
             transaction_hash = transaction.transaction_hash()
             
-            logging.debug("Rewinding transaction with {} inputs and {} "
-                "outputs".format(len(transaction.inputs), 
-                len(transaction.outputs)))
-            
             self.remove_transaction(transaction)
             
     def get_unused_outputs(self):
@@ -1128,7 +1120,6 @@ class Wallet(object):
                         if self.spendable.has_unused_output(spent_output):
                             # This output we had available got spent
                             self.spendable.remove_unused_output(spent_output)
-                            logging.debug("Our available output was spent")
                     
                     for i, output in enumerate(transaction.outputs):
                         # Unpack each output tuple
@@ -1138,7 +1129,6 @@ class Wallet(object):
                             # This output is spendable by us           
                             self.spendable.add_unused_output((transaction_hash,
                                 i, amount, destination))
-                            logging.debug("New available output!")
                                 
                 # Re-set our set of transactions we should draw upon to all the
                 # transactions available
@@ -1161,7 +1151,6 @@ class Wallet(object):
                         if self.keystore.has_key(spent_output[3]):
                             # This output we spent got unspent
                             self.spendable.add_unused_output(spent_output)
-                        logging.debug("Our spent output is available again")
                     
                     for i, output in enumerate(transaction.outputs):
                         # Unpack each output tuple
@@ -1175,7 +1164,6 @@ class Wallet(object):
                             # This output we had available to spend got un-made        
                             self.spendable.remove_unused_output(spent_output)
                             
-                            logging.debug("Lost available output!")
                             
                 # Re-set our set of transactions we should draw upon to all the
                 # transactions available
